#
# Copyright (c) 2020-2021 Arm Limited and Contributors. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
#
"""Exceptions raised by mbed tools."""


class ToolsError(Exception):
    """Base class for tools errors."""
