#
# Copyright (c) 2020-2021 Arm Limited and Contributors. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
#
"""mbed_tools command line interface."""

from mbed_tools.cli.main import cli, LOGGER
